from distutils.core import setup

setup(
  name = 'bnlp',
  packages = ['bnlp'],
  package_dir = {'bnlp': 'bnlp'},
  package_data = {'bnlp': [ '*.py', 'data/demonyms.txt', 'data/non-locations.txt', 'data/keywords/orgs/*.txt' ]},
  version = '0.8',
  description = 'Better Natural Language Processing, a collection of open source nlp methods used by Better Know The Opposition',
  author = 'Better Know The Opposition',
  author_email = 'hello@betterknowtheopposition.com',
  url = 'https://github.com/BKTO/bnlp',
  download_url = 'https://github.com/BKTO/bnlp',
  keywords = ['nlp'],
  classifiers = [],
)
